
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: AcceptGroup, IsInInstance, LFGInvitePopup, next, QueueStatusButton, StaticPopup_Hide, StaticPopupSpecial_Hide, string, strlower, table, type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Automation Invite")
local Social = LibStub("SixSocial-1.0")

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

-- Get the key name for an ignore target.
local function GetTargetKey(target)
	return strlower(string.split("-", target or ""))
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
	self:RegisterEvent("GROUP_JOINED")
	self:RegisterEvent("PARTY_INVITE_REQUEST")
end

-- Get a unit's friend name (BattleTag or character name).
function M:GetFriendName(unit)
	local friend = Social:GetFriendByUnit(unit)
	return friend and (friend.battleTag or friend.characterName) or nil
end

-- Get a unit's club rank.
function M:GetClubRank(unit)
	for club in Social:IterateClubs() do
		for member in club:IterateMembers() do
			if member.unit == unit then
				return member.rank
			end
		end
	end
end

-- Check whether a unit was whispered.
function M:WasWhispered(unit)
	return self.whisperUnit == unit
end

-------------------------------------------------------------------------------
-- Ignore List
-------------------------------------------------------------------------------

function M:GetIgnoreList()
	local ignoreList = {}

	for _, entry in next, self.config.blacklist do
		ignoreList[entry.target] = entry.target
	end

	return ignoreList
end

function M:Ignore(target)
	if not self:IsIgnored(target) then
		table.insert(self.config.blacklist, {
			key = GetTargetKey(target),
			target = target,
		})
	end
end

function M:IsIgnored(target)
	if type(target) == "string" then
		for index, entry in next, self.config.blacklist do
			if GetTargetKey(target) == entry.key then
				return index
			end
		end
	end

	return false
end

function M:Unignore(target)
	local index = self:IsIgnored(target)

	if index then
		table.remove(self.config.blacklist, index)
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

-- Set the last whispered unit.
function M:CHAT_MSG_WHISPER_INFORM(event, text, playerName)
	self.whisperUnit = A.FormatUnit(playerName)
end

-- Hide the party invite popup when joining a group.
function M:GROUP_JOINED()
	StaticPopup_Hide("PARTY_INVITE")

	if LFGInvitePopup then
		StaticPopupSpecial_Hide(LFGInvitePopup)
	end
end

-- Accept a group invitation if the sender is known and the player is not in a queue or instance.
function M:PARTY_INVITE_REQUEST(event, name)
	if (not name) then return end
	if (self.config.disableInQueue and QueueStatusButton:IsVisible()) then return end
	if (self.config.disableInInstance and IsInInstance()) then return end

	local unit = A.FormatUnit(name)
	local clubRank = self:GetClubRank(unit)
	local friendName = self:GetFriendName(unit)
	local wasWhispered = self:WasWhispered(unit)

	for _, target in next, {clubRank, friendName, unit} do
		if self:IsIgnored(target) then
			return
		end
	end

	if (self.config.allowClubMembers and clubRank)
		or (self.config.allowFriends and friendName)
		or (self.config.allowLastCharacterWhispered and wasWhispered)
	then
		AcceptGroup()
	end
end
